@@ -278,8 +278,6 @@ Mode = function(x) {
 
 #' Random walk
 #'
-#' Internal soundgen function.
-#'
 #' Generates a random walk with flexible control over its range, trend, and
 #' smoothness. It works by calling \code{\link[stats]{rnorm}} at each step and
 #' taking a cumulative sum of the generated values. Smoothness is controlled by
@@ -301,13 +299,12 @@ Mode = function(x) {
 #'   with an upward and a downward curve. Larger absolute values of trend
 #'   produce less and less random behavior
 #' @return Returns a numeric vector of length len and range from 0 to rw_range.
-#' @keywords internal
+#' @export
 #' @examples
-#' plot(soundgen:::getRandomWalk(len = 1000, rw_range = 5,
-#'   rw_smoothing = .2))
-#' plot(soundgen:::getRandomWalk(len = 1000, rw_range = 15,
+#' plot(getRandomWalk(len = 1000, rw_range = 5, rw_smoothing = .2))
+#' plot(getRandomWalk(len = 1000, rw_range = 15,
 #'   rw_smoothing = .2, trend = c(.5, -.5)))
-#' plot(soundgen:::getRandomWalk(len = 1000, rw_range = 15,
+#' plot(getRandomWalk(len = 1000, rw_range = 15,
 #'   rw_smoothing = .2, trend = c(15, -1)))
 getRandomWalk = function(len,
                          rw_range = 1,
@@ -315,7 +312,7 @@ getRandomWalk = function(len,
                          method = c('linear', 'spline')[2],
                          trend = 0) {
   if (len < 2)
-    return (rgamma(1, 1 / rw_range ^ 2, 1 / rw_range ^ 2))
+    return(rgamma(1, 1 / rw_range ^ 2, 1 / rw_range ^ 2))
 
   # generate a random walk (rw) of length depending on rw_smoothing,
   # then linear extrapolation to len
@@ -345,55 +342,65 @@ getRandomWalk = function(len,
   # normalize
   rw_normalized = rw_long - min(rw_long)
   rw_normalized = rw_normalized / max(abs(rw_normalized)) * rw_range
-  return (rw_normalized)
+  return(rw_normalized)
 }
 
 
 #' Discrete random walk
 #'
-#' Internal soundgen function.
-#'
 #' Takes a continuous random walk and converts it to continuous epochs of
 #' repeated values 0/1/2, each at least minLength points long. 0/1/2 correspond
 #' to different noise regimes: 0 = no noise, 1 = subharmonics, 2 = subharmonics
 #' and jitter/shimmer.
-#' @keywords internal
 #' @param rw a random walk generated by \code{\link{getRandomWalk}} (expected
 #'   range 0 to 100)
 #' @param nonlinBalance a number between 0 to 100: 0 = returns all zeros;
 #'   100 = returns all twos
 #' @param minLength the mimimum length of each epoch
+#' @param q1,q2 cutoff points for transitioning from regime 0 to 1 (q1) or from
+#'   regime 1 to 2 (q2). See noiseThresholdsDict for defaults
 #' @param plot if TRUE, plots the random walk underlying nonlinear regimes
 #' @return Returns a vector of integers (0/1/2) of the same length as rw.
-#' @keywords internal
+#' @export
 #' @examples
-#' rw = soundgen:::getRandomWalk(len = 100, rw_range = 100, rw_smoothing = .2)
-#' r = soundgen:::getIntegerRandomWalk(rw, nonlinBalance = 75,
-#'                                      minLength = 10, plot = TRUE)
-#' r = soundgen:::getIntegerRandomWalk(rw, nonlinBalance = 15,
-#'                                      minLength = 10, plot = TRUE)
+#' rw = getRandomWalk(len = 100, rw_range = 100, rw_smoothing = .2)
+#' r = getIntegerRandomWalk(rw, nonlinBalance = 75,
+#'                          minLength = 10, plot = TRUE)
+#' r = getIntegerRandomWalk(rw, nonlinBalance = 15,
+#'                          q1 = 30, q2 = 70,
+#'                          minLength = 10, plot = TRUE)
 getIntegerRandomWalk = function(rw,
                                 nonlinBalance = 50,
                                 minLength = 50,
+                                q1 = NULL,
+                                q2 = NULL,
                                 plot = FALSE) {
   len = length(rw)
-  if (nonlinBalance == 0) return(rep(0, len))
-  if (nonlinBalance == 100) return(rep(2, len))
 
   # calculate thresholds for different noise regimes
-  q1 = noiseThresholdsDict$q1[nonlinBalance + 1]
-  # +1 b/c the rows indices in noiseThresholdsDict start from 0, not 1
-  q2 = noiseThresholdsDict$q2[nonlinBalance + 1]
+  if (is.null(q1)) {
+    q1 = noiseThresholdsDict$q1[nonlinBalance + 1]
+    # +1 b/c the rows indices in noiseThresholdsDict start from 0, not 1
+  }
+  if (is.null(q2)) {
+    q2 = noiseThresholdsDict$q2[nonlinBalance + 1]
+  }
 
-  # convert continuous rw to discrete epochs based on q1 and q2 thresholds
-  rw_bin = rep(0, len)
-  rw_bin[which(rw > q1)] = 1
-  rw_bin[which(rw > q2)] = 2   # plot (rw_bin, ylim=c(0,2))
+  if (nonlinBalance == 0) {
+    rw_bin = rep(0, len)
+  } else if (nonlinBalance == 100) {
+    rw_bin = rep(2, len)
+  } else {
+    # convert continuous rw to discrete epochs based on q1 and q2 thresholds
+    rw_bin = rep(0, len)
+    rw_bin[which(rw > q1)] = 1
+    rw_bin[which(rw > q2)] = 2   # plot (rw_bin, ylim=c(0,2))
 
-  # make sure each epoch is long enough
-  rw_bin = clumper(rw_bin, minLength = minLength)
+    # make sure each epoch is long enough
+    rw_bin = clumper(rw_bin, minLength = minLength)
+  }
 
-  if (FALSE) {
+  if (plot) {
     rw_bin_100 = rw_bin
     rw_bin_100[rw_bin_100 == 1] = q1
     rw_bin_100[rw_bin_100 == 2] = q2
@@ -512,19 +519,20 @@ matchColumns = function (matrix_short, nCol, padWith = 0) {
 #' @param v1,v2 numeric vectors
 #' @param insertionPoint the index of element in vector 1 at which vector 2 will
 #'   be inserted (any integer, can also be negative)
+#' @param normalize if TRUE, the output is normalized to range from -1 to +1
 #' @export
 #' @examples
 #' v1 = 1:6
 #' v2 = rep(100, 3)
-#' addVectors(v1, v2, insertionPoint = 5)
-#' addVectors(v1, v2, insertionPoint = -4)
+#' addVectors(v1, v2, insertionPoint = 5, normalize = FALSE)
+#' addVectors(v1, v2, insertionPoint = -4, normalize = FALSE)
 #' # note the asymmetry: insertionPoint refers to the first arg
-#' addVectors(v2, v1, insertionPoint = -4)
+#' addVectors(v2, v1, insertionPoint = -4, normalize = FALSE)
 #'
 #' v3 = rep(100, 15)
-#' addVectors(v1, v3, insertionPoint = -4)
-#' addVectors(v2, v3, insertionPoint = 7)
-addVectors = function(v1, v2, insertionPoint = 1) {
+#' addVectors(v1, v3, insertionPoint = -4, normalize = FALSE)
+#' addVectors(v2, v3, insertionPoint = 7, normalize = FALSE)
+addVectors = function(v1, v2, insertionPoint = 1, normalize = TRUE) {
   if (!is.numeric(v1)) stop(paste('Non-numeric v1:', head(v1)))
   if (!is.numeric(v2)) stop(paste('Non-numeric v2:', head(v2)))
   v1[is.na(v1)] = 0
@@ -551,8 +559,8 @@ addVectors = function(v1, v2, insertionPoint = 1) {
 
   # add and normalize
   out = v1 + v2
-  out = out / max(abs(out))
-  return (out)
+  if (normalize) out = out / max(abs(out))
+  return(out)
 }
 
 
@@ -716,3 +724,38 @@ getSigmoid = function(len,
   return(out)
 }
 
+
+#' Report CI
+#'
+#' Internal soundgen function
+#'
+#' Takes a numeric vector or matrix with three elements / columns: fit, lower
+#' quantile from a CI, and upper quantile from a CI. For each row, it prints the
+#' result as fit and CI in square brackets
+#' @param n numeric vector or matrix
+#' @param digits number of decimal points to preserve
+#' @keywords internal
+#' @examples
+#' n = rnorm(100)
+#' soundgen:::reportCI(quantile(n, probs = c(.5, .025, .975)))
+#'
+#' a = data.frame(fit = c(3, 5, 7),
+#'                lwr = c(1, 4, 6.5),
+#'                upr = c(5, 6, 7.1))
+#' soundgen:::reportCI(a, 1)
+reportCI = function(n, digits = 2) {
+  if (class(n) == 'data.frame') {
+    n = as.matrix(n)
+  }
+  n = round(n, digits)
+  if (class(n) == 'matrix') {
+    out = matrix(NA, nrow = nrow(n))
+    rownames(out) = rownames(n)
+    for (i in 1:nrow(n)) {
+      out[i, ] = reportCI(n[i, ])
+    }
+    return(out)
+  } else {
+    paste0(n[1], ' [', n[2], ', ', n[3], ']')
+  }
+}
